pro sector_avg,image,index=index,origin=origin,xxx,sig,rr,fullsig,$
	xy=xy,new=new,fit_param=fit_param,yfit=yfit,$
	noplot=noplot,multiple=multiple,unc_data=unc_data,$
	unc_sig=unc_sig,unc_fullsig=unc_fullsig,skip=skip,$
	sector=sector,noqx=noqx,sectparam=sectparam,azipix=azipix
;+
; NAME
;       sector_avg
; PURPOSE
;	To compute the azimuthally-averaged radial intensity distribution from
;	some origin -- either the center of the sun or an input location.
;	Radial totals, not averaged over azimuth, can be returned also for
;	use in SXT_TEEM which requires actual total DN if error bars are
;	be put on the temperatures.
;	   Signal averages or totals over specified sectors also an option.
; CALLING SEQUENCE
;	sector_avg,image,index=index,origin=origin,xxx,sig,rr,fullsig,$
;         xy=xy,new=new,fit_param=fit_param,yfit=yfit,$
;         noplot=noplot,multiple=multiple,unc_data=unc_data,$
;         unc_sig=unc_sig,unc_fullsig=unc_fullsig,skip=skip,$
;         sector=sector,noqx=noqx,sectparam=sectparam,azipix=azipix
;       sector_avg,image,index=index,xxx,sig
;	sector_avg,image,origin=[361,400],xxx,sig, $
;		fit_param=fit_param,yfit=yfit,multiple=2
;	sector_avg,image,index=index,xxx,sig,rr,/new,/origin,/noplot
; INPUT
;       image - Fully corrected image.
;	Either:  index, origin or both.
; OUTPUT TO SCREEN
;	If the keyword sectparam is not set then output of the following
;	form is written to the screen:
;		The slope of the fit is     -2.03562
;		Fitted expression:   Signal =  6.6e+06*R^(-2.04)   per pixel.
; OPTIONAL OUTPUT
;	xxx - linear array of R values for plotting
;	sig - array of signal values, averaged over azimuth, function of R
;		units are DN/pixel for pixel size in image
;		Used for plotting signal vs. R
;       rr - array of radii of all distances from origin
;		units are pixels (pixel size of image)
;       xy - array of pixels included in the analysis
;	unc_sig - compression uncertainties on azimuthly averaged values
;	fullsig - azimuthal totals, not averaged over azimuth, function of R
;		for use with SXT_TEEM
;	unc_fullsig - compression errors on azimuth totals
; OPTIONAL INPUT
;	rr, return from first run and input for subsequent runs 
;		to save time.
; OPTIONAL KEYWORD INPUT
;	index - to permit getting suncenter from the data base
;		and labelling of output figures
;	origin - origin of coordinates for azimuth averaging, if this is
;		 specified it takes priority over suncenter from index
;		 but one or the other must be provided.
;
;	Note: 	origin = [x0,y0] chooses x0,y0 as origin of coordinates.
;	      	/origin allows choosing origin by click on image.
;
;	xy - array defining target area,
;	     	default is to read from image with WDEFROI
;       /new - set if rr array exists but you wish to recompute it
;		forces recomputation of xy also.
;	fit_param - set this to execute and plot a log10-log10 fit 
;		to xxx vs. sig and, if you wish, return the parameters
;               Set fit_param=1 to return the parameters.
;	yfit - values of the fitted log10-log10 curve at xxx
;	/noplot - causes program to skip plotting of raw data, saves time.
;	multiple - Use for 2 or more disconnected regions (e.g., multiple=3)
;	unc_data - array of uncertainties of values in image from SXT_PREP
;	skip - array of indices to be excluded from analysis.  This is
;		a good way to eliminate unwanted features that can be
;		identified with a "where" analysis before using sector_avg.
;	sector = [begin_angle,end_angle,angle_step]
;		measured clockwise in degrees from CCD vertical
;	/noqx - causes the program to complete without pauses and questions.
;               and shows no data plots.
; OPTIONAL KEYWORD OUTPUT
;	azipix =  number of pixels at each R position.
;	    Needed for renormalizing emission measure after SXT_TEEM is run.
;	sectparam = structure with angles, fitting parameters, and number
;	    of valid pixels in each sector for sector analysis.  The parameter
;	    sectparam.angle refers to the middle of the sector.
;		** Structure <45f20088>, 4 tags, length=16, refs=2:
;		   ANGLE           FLOAT           228.000
;		   CONST           FLOAT           6.78359
;		   SLOPE           FLOAT          -1.99499
;		   NUMPIX          INT            146 
; PROGRAMS CALLED
;	sfd_comp, cursor, plots, plusmark, gt_corner, wdefroi, where,
;	findgen, rebin, reform, sqrt, wdef, xyouts, plot, fix, max,
;	total, float, n_elements, indgen, gt_res, alog10, poly_fit, print
;	sector_bound
; HISTORY
;       LWA 26-June-94, wrote original program.
;	LWA 2-Dec-94, added capability for non-suncenter origin.
;	LWA 26-Jan-95, cleaned up header and fixed /new omission.
;		Changed order of variables in call.
;	LWA 29-Jan-95, added keyword for disconnected regions.
;	LWA 16-Mar-95, replaced sxt_cen with -gt_corner(index,/from_sc)
;	        and made index a keyword.
;	LWA 23-Mar-95, Corrected to include negative values in averages.
;		and added keyword /noplot.
;	LWA 6-Apr-95, Added computation of uncertainties.  Eliminated "img".
;	LWA 8-Apr-95, Added fullsig and unc_fullsig and changed "out" to "sig".
;	LWA 1-Jun-95, Added skip keyword.
;	LWA 14-Jun-95, Changed name to AZIMUTH_AVG.PRO
;		Added sector averaging (What a mess!)
;	LWA 5-Jul-95, Corrected header.
;	LWA 28-nov-95, Added noqx keyword.
;	LWA 29-Nov-95, Fixed array dimensions and fit_param for sectoring.
;	LWA 30-Nov-95, Fixed rmax-rmin dimensioning problem.
;	LWA 25-Oct=96, Fixed problem of missing nxy if no skip.
;	LWA  6-Apr-01, Added sector parameter, sectparam, output.
;	LWA  8-Apr-01, Added numpix to sectparam.
;		       Added test for sectors with no data.
;		       Incorporated keyword azipix, making AZIMUTH_AVG obsolete.
;	LWA 23-May-01, Split if test on fitparam and nii to avoid error when
;		       not fitting data.  Added needed fitparam test.
;	LWA 25-May-01, Moved sectparam.numpix out of fitparam check.
;	LWA 28-May-01, Updated header.
;-

if (NOT keyword_set(origin) and n_elements(index) eq 0) then begin
   print
   print,' SECTOR_AVG:  You must provide either INDEX or ORIGIN to define'
   print,'              the origin of radial coordinates.'
   print,' Returning ...'
   print
   tbeep, 3
   return
endif

;  Initial setup of output arrays.

xxx=0.
sig=0.
azipix=0L
unc_sig=0.
fullsig=0.
unc_fullsig=0.

;  Open a window in the upper right

siz=size(image)
nx=siz(1)
ny=siz(2)
wdef,28,nx,ny

;  Determine origin of coordinates

if keyword_set(origin) then begin
   if (n_elements(origin) eq 1) then begin
	tvscl,sfd_comp(image)
	;  Click on image to designate origin      
	print, '* Use left button to mark origin until you are satisfied'
	print, '* Use the right button to continue'
	qdone = 0
	while not qdone do begin
	    wait, 0.3
	    cursor, xx, yy, /device
	    button = !err
	    case button of
	        1: begin        ;left button
	                print, 'Center location: ', xx, yy
			x=xx & y=yy
	           end
	        4: begin        ;right button
	           end
	    endcase
	    if (button ne 4) then begin
	         plots, [x-5,x-2],[y,y],/device
	         plots, [x,x],[y+2,y+5],/device
	    endif
	    if (button eq 4) then qdone = 1
	endwhile
	plusmark,x,y
   endif else begin
	x=origin(0) & y=origin(1)
   endelse
endif else begin
   suncenter=-gt_corner(index,/from_sc)
   x=suncenter(0)
   y=suncenter(1)
endelse

;  Insert cross at origin into image array

image(x-3:x+3,y)=0
image(x,y-3:y+3)=0

;  If designated area for analysis is not input then get it with wdefroi

if NOT keyword_set(sector) then begin
   if n_elements(xy) lt 2 or keyword_set(new) then begin 
      xy=wdefroi(image,/logscale)
      if keyword_set(multiple) then begin
         for i=0,multiple-2 do begin
   	    timg=image
            timg(xy)=1
            xyt=wdefroi(timg,/logscale)
   	    xy=[xy,xyt]
         endfor
      endif
   endif else begin
      tvscl,sfd_comp(image)
      xy=xy 
   endelse
   if keyword_set(skip) then begin   ; Get rid of unwanted pixels.
      ok=bytarr(nx,ny)
      ok(xy)=1
      ok(skip)=0
      xy=where(ok)
   endif
endif

xx=findgen(nx)
yy=findgen(ny)

;  Compute the array of radii if necessary

if (keyword_set(new) or n_elements(rr) eq 0) then begin

      xx=(findgen(nx)-x)^2
      xx=rebin(xx,nx,ny)
      yy=(findgen(ny)-y)^2
      yy=reform(yy,1,ny)
      yy=rebin(yy,nx,ny)
      rr=sqrt(temporary(xx)+temporary(yy))

      if keyword_set(skip) then rr(skip)=0

endif else begin
   rr=rr
endelse

;  Set up for sector averaging

if keyword_set(sector) then begin
   nsector=fix((sector(1)-sector(0))/sector(2))	
   angle=findgen(nsector)*sector(2)+sector(0)+(sector(2)/2.)	;8-apr-2001
   sectparam=create_struct('angle',0.0,'const',0.0,$
	'slope',0.0,'numpix',0)					;6-apr-2001
   sectparam=replicate(sectparam,nsector)			;6-apr-2001
endif else begin
   nsector=1
endelse

temp=sfd_comp(image)
if keyword_set(skip) then temp(skip)=0
tvscl,temp
rmax=0
rmin=1024
rmin2=1024

for is=0,nsector-1 do begin

   if keyword_set(sector) and rmax eq 0 then begin
      for j=0,nsector-1 do begin     ; Determine the length of output arrays.
         xy=sector_bound(sector(0)+(j)*sector(2),$
            sector(0)+(j+1)*sector(2),x,y,nx,ny)
         rmax2=max(rr(xy))
         if rmax2 gt rmax then rmax=rmax2
         no0=where(rr(xy) ne 0,no0n)
         if no0n gt 0 then rmin2=min(rr(xy(no0)))
         if rmin2 lt rmin then rmin=rmin2
         temp=bytarr(nx,ny)
         temp(xy)=1
         ocontour,temp,levels=0.5
      endfor
      if NOT keyword_set(noqx) then begin
         yesnox,'ARE YOU SATISFIED WITH THE SECTORS',in,'yes'
         if in eq 0 then return
      endif
      mmm=fix(rmax-rmin+1.0)
      xxx2=intarr(mmm,nsector)
      sig2=fltarr(mmm,nsector)
      azipix2=lonarr(mmm,nsector)
      fullsig2=fltarr(mmm,nsector)
      unc_sig2=fltarr(mmm,nsector)
      unc_fullsig2=fltarr(mmm,nsector)
      if keyword_set(fit_param) then begin
         fits=fltarr(2,nsector)
         fit_param=[0.0,0.0]	;Dumb fix to remove "empty sector" problem.
      endif
   endif

   if keyword_set(sector) then begin   
      xy=sector_bound(sector(0)+(is)*sector(2),sector(0)+(is+1)*sector(2),x,y,nx,ny)
      if keyword_set(skip) then begin
         ok=bytarr(nx,ny)
         ok(xy)=1
         ok(skip)=0
         xy=where(ok,nxy)
      endif else begin
         nxy=n_elements(xy)
      endelse
   endif
      
; Display the pixels to be included in the analysis.
;  Open a window in the upper left

   if NOT keyword_set(sector) then nxy=1
   if (nxy gt 0) then begin
      timg=fltarr(nx,ny)
      timg(xy)=image(xy)
      if NOT keyword_set(noqx) then begin
          wdef,30,nx,ny
         if NOT keyword_set(noplot) then begin
            tvscl,sfd_comp(timg)
            plusmark,x,y
            xyouts,/dev,0,0,'Area included in analysis',charsize=1.4
         endif
         if NOT keyword_set(noplot) then begin
            ;  Open a second window in the upper right and plot the data
            wdef,24
            tit=string('Number of data points =',n_elements(xy))
            plot_oo,rr(xy),image(xy)>1,psym=3,title=tit,$  
            	xtitle='Radius (pixels)                   
            print
            print, '  Here is a plot of the positive data points.'
            print, '  ..Not to worry, the negatives are in the averages properly.'
         endif
         print
         print, 'PATIENCE -- it takes a long time to do the averages.'
      endif

   ;  Form the azimuth averages

      rrf=fix(rr(xy))
      values=image(xy)
      first_r=min(rrf)
      last_r=max(rrf)
      nn=last_r-first_r+1
      sig=fltarr(nn)
      azipix=lonarr(nn)
      fullsig=fltarr(nn)
      jj=bytarr(nn)+1

      for i=0,nn-1 do begin
         ii=where(rrf eq first_r+i)
         if (ii(0) ne (-1)) then begin
            fullsig(i)=total(values(ii))
            sig(i)=fullsig(i)/(n_elements(ii))
            azipix(i)=n_elements(ii)
         endif else begin
            jj(i)=0 & azipix(i)=0 & fullsig(i)=0 & sig(i)=0
         endelse
      endfor

   ;  Form the uncertainties on the azimuth averages

      if keyword_set(unc_data) then begin
      unc=float(unc_data(xy))
      unc_sig=fltarr(nn)
      unc_fullsig=fltarr(nn)
         for i=0,nn-1 do begin
           ii=where(rrf eq first_r+i)
             if (ii(0) ne (-1)) then begin
               unc_fullsig(i)=sqrt(total(unc(ii)^2))
               unc_sig(i)=unc_fullsig(i)/(n_elements(ii))
             endif 
         endfor
      endif

   ;  Display the averaged data
   ;  Open a window in the lower right
      xxx=indgen(nn)+first_r
      jji=where(jj ne 0)
      xxx=xxx(jji)
      sig=sig(jji)

      if NOT keyword_set(noqx) then begin
         wdef,29
         ii=where(sig gt 1) ; Eliminate low signals so plot scales nicely.
         plot_oo,xxx(ii),sig(ii),psym=3,xtitle='Radius (pixels)',$
            ytitle='Signal (DN/pix for image)',$
            title='Average Radial Intensity'
         if n_elements(index) ne 0 then begin
            xyouts,/norm,.5,.92,siz=1.2, $
         	'Filter: '+gt_filtb(index,/st)
            xyouts,/norm,.5,.89,siz=1.2, $
         	'Time: '+fmt_tim(index)
            xyouts,/norm,.5,.86,'Radius:  '+gt_res(index,/str)+' resolution pixels.'
         endif
      endif
      if keyword_set(fit_param) then begin
         ;  Fit the averaged data
         ii=where(sig gt 0,nii) ; Power law fit blows up with negative data.
         if nii gt 0 then begin
            fit_param=poly_fit(alog10(xxx(ii)),alog10(sig(ii)),1,yfit)
            if NOT keyword_set(noqx) then oplot,xxx(ii),10^yfit,color=180
            if NOT keyword_set(sectparam) then begin
               print,'The slope of the fit is',fit_param(1)

     ; Write out the fitted function.

              print,'Fitted expression:   Signal ='$
              +string(format='(e9.1)',10^fit_param(0))$
              +'*R^('+string(format='(f5.2)',fit_param(1))+')   per pixel.'
            endif
         endif
      endif

      if keyword_set(sector) then begin
         xxx2(0,is)=xxx
         sig2(0,is)=sig
         azipix2(0,is)=azipix
         fullsig2(0,is)=fullsig
         unc_sig2(0,is)=unc_sig
         unc_fullsig2(0,is)=unc_fullsig
         if keyword_set(fit_param) then begin	;23-May-2001
            if nii gt 0 then fits(*,is)=fit_param $
		else fits(*,is)=[0,0]
         endif
      endif

      if keyword_set(sector) then begin
         if NOT keyword_set(noqx) then begin
            yesnox,'DO YOU WANT TO CONTINUE TO LOOK AT SECTORS?',in,'yes'
            if in eq 0 then goto, closeout
            wset,30
         endif
         if keyword_set(noqx) then print,'Sector',is,' ','is finished.'
         sectparam(is).numpix=n_elements(xy)    ;8-Apr-2001
         if keyword_set(fitparam) then begin    ;23-May=2001
            sectparam(is).const=fit_param(0)	;6-Apr-2001 
            sectparam(is).slope=fit_param(1)	;6-Apr-2001 
         endif
      endif
   endif
   sectparam(is).angle=angle(is)       		;8-Apr-2001 

ENDFOR   ; End of sector loop.
      
closeout :
if keyword_set(sector) then begin      
      xxx=xxx2
      sig=sig2
      azipix=azipix2
      fullsig=fullsig2
      unc_sig=unc_sig2
      unc_fullsig=unc_fullsig2
      if keyword_set(fit_param) then fit_param=fits
endif

end


